#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
PURPOSE: Models for Meetup.com api
AUTHOR: dylangregersen
DATE: Mon Sep 15 00:12:21 2014
"""
# ########################################################################### #

# import modules 

from __future__ import print_function, division
from django.db import models
import datetime
import pytz
import warnings
from meetup.sync_utils import (fro_meetup_geo,to_meetup_geo,
                              fro_meetup_timestamp,to_meetup_timestamp)

STATUS_CHOICES = ('past','pending','upcoming')
VISIBILITY_CHOICES = ('public','private')

# ########################################################################### #

class Mapper (object):
    """ Used to map names in two directions """

    def __init__ (self,name=""):
        self._to = {}
        self._from = {}
        self.name = name
        
    def __setitem__ (self,key,value):
        self._to[key] = value
        self._from[value] = key
        
    def __getitem__ (self,key):
        return self.to(key)
    
    def keys (self):
        return self._to.keys()
    
    def values (self):
        return self._to.values()
    
    def to (self,key):
        return self._to[key]

    def get_to (self,key,default=None):
        return self._to.get(key,default)

    def fro (self,value):
        return self._from[value]
        
    def get_fro (self,value,default=None):
        return self._from.get(value,default)
      
    def __iter__ (self):
        return iter(self._to.items()) 

class MeetupManager (models.Manager):  
          
    def _object_to_meetup_data (self,obj):
        mapper = self.meetup_mapper
        kws = {}
        field_names = [f.name for f in self.model._meta.fields]
        for field in field_names:            
            meetup_key = mapper.get_to(field,field)                
            kws[meetup_key] = getattr(obj,field)
        return self._post_object_to_meetup_data(obj,kws)
            
    def _post_object_to_meetup_data (self,obj,kws):
        return kws
        
    def _meetup_data_to_kws (self,meetup_data):
        mapper = self.meetup_mapper
        kws = {}
        field_names = [f.name for f in self.model._meta.fields]        
        for meetup_key in meetup_data:
            # map the meetup_key to the field name
            field = mapper.get_fro(meetup_key,meetup_key)
            # check the field name
            if field not in field_names:
                warnings.warn("ignoring meetup_key '{}' as field '{}'".format(meetup_key,field))
                continue
            # get the field values
            kws[field] = meetup_data[meetup_key]
        return self._post_meetup_data_to_kws(meetup_data,kws)
      
    def _post_meetup_data_to_kws (self,meetup_data,kws):
        return kws
     
    def _post_object_creation_or_update (self,obj,md):
        return obj
                
    def to_meetup_data (self,**filter):        
        """ Takes filters and converts each to meetup data """                
        objs = self.filter(**filter)
        return [self._object_to_meetup_data(obj) for obj in objs]
      
    def from_meetup_data (self,meetup_data,sync=True):
        """ Takes Meetup data to model data
        
        Parameters
        meetup_data_kws : dict or list of dict
        sync : bool
            sync objects
            
        Parameters
        objects : list of objects OR list of dictionaries
            If sync is False then returns a list of the dictionary key/values to
            use to create or update the objects
            
        """            
        if isinstance(meetup_data,dict):
            meetup_data = [meetup_data]
        
        # primary key field name        
        pk_field = self.model._meta.pk.name
                
        objects = []
        for md in meetup_data:
            # get the key/value data from the meetup_data
            kws = self._meetup_data_to_kws(md)
            if sync:
                # create/update the group
                try:
                    obj = self.get(pk=kws[pk_field])
                    for key in kws:
                        setattr(obj,key,kws[key])
                    obj.save()
                except self.model.DoesNotExist:
                    obj = self.create(**kws)
                obj = self._post_object_creation_or_update(obj,md)
            else:
                # pass the key/value data through
                obj = kws
            objects.append(obj)
        return obj

pass
# ########################################################################### #

class VenueManager (MeetupManager):
    
    meetup_mapper = Mapper("venue_model -> meetup_data")
    
    def _post_meetup_data_to_kws (self,obj,kws):
        # convert longitude and latitude
        for key in ('lon','lat'):
            loc = fro_meetup_geo(kws.pop(key))
            if loc is not None:
                kws[key] = loc
        return kws
    
    def _post_object_to_meetup_data (self,meetup_data,kws):
        # convert longitude and latitude
        for key in ('lon','lat'):
            loc = to_meetup_geo(kws.pop(key))
            if loc is not None:
                kws[key] = loc
        return kws        
   
class Venue (models.Model):
    """ Meetup Venue Model """

    objects = VenueManager()
        
    name = models.CharField(max_length=128)
    city = models.CharField(max_length=128,blank=True)
    state = models.CharField(max_length=128,blank=True)
    country = models.CharField(max_length=128,blank=True)
    
    address_1 = models.CharField(max_length=128,blank=True)
    address_2 = models.CharField(max_length=128,blank=True)
    address_2 = models.CharField(max_length=128,blank=True)    
    
    lat = models.FloatField(blank=True)
    lon = models.FloatField(blank=True)
          
    def __unicode__ (self):
        return self.name 

class Member (models.Model):
    """ Meetup member account """
    member_id = models.IntegerField(primary_key=True)
    name = models.CharField(max_length=255, blank=True)        
    bio = models.TextField(blank=True)
    status = models.CharField(max_length=33)
    created = models.DateTimeField()   
    updated = models.DateTimeField()
    visited = models.DateTimeField()    
    profile_url = models.URLField(max_length=255, blank=True) 
    
    def __unicode__(self):
        return self.name

class GroupManager (MeetupManager):
    
    meetup_mapper = Mapper("group_model -> meetup_data")
    meetup_mapper['n_members'] = 'members'
        
    def _post_meetup_data_to_kws (self,meetup_data,kws):
        # convert longitude and latitude
        for key in ('lon','lat'):
            loc = fro_meetup_geo(kws.pop(key,None))
            if loc is not None:
                kws[key] = loc      
        # if there are members then convert that number to an int
        key = str('n_members')
        if key in kws:
            kws[key] = int(kws[key] or 0)             
        
        # if timezone then check and convert to timezone string
        key = 'timezone'
        if key in kws:
            kws[key] = str(pytz.timezone(meetup_data[key])) 
                              
        return kws
    
    def _post_object_to_meetup_data (self,obj,kws):
        # convert longitude and latitude
        for key in ('lon','lat'):
            loc = to_meetup_geo(kws.pop(key))
            if loc is not None:
                kws[key] = loc
        kws['members'] = str(kws['members'])
        return kws        
                   
class Group (models.Model):
    """ Meetup Group Model """
    objects = GroupManager()
    
    VISIBLITY_CHOICES = VISIBILITY_CHOICES
    
    name = models.CharField(max_length=128)
    urlname = models.CharField(max_length=128)
    city = models.CharField(max_length=128,blank=True)
    state = models.CharField(max_length=128,blank=True)
    country = models.CharField(max_length=128,blank=True)
    link = models.URLField()
    visibility = models.CharField(max_length=128,help_text="Visiblity to the users")
    timezone = models.CharField(max_length=128,blank=True)    
    lat = models.FloatField(blank=True)
    lon = models.FloatField(blank=True)    
    n_members = models.IntegerField(default=0,blank=True) 
    who = models.CharField(max_length=128,blank=True)
    # join_mode = ('open',...)
    # category = ForeignKey   
    # topics = ForeignKey
    # photos = ...
    # organizer_id = ForeignKey
       
    def __unicode__ (self):
        return self.name
        
class EventManager(MeetupManager):
        
    meetup_mapper = Mapper("event_model_field -> meetup_data_key")
    meetup_mapper['event_timestamp'] = 'time'
        
    def _post_meetup_data_to_kws (self,meetup_data,kws):
        group_data = meetup_data['group']
        kws['group'] = Group.objects.from_meetup_data(group_data,sync=True)
        tzinfo = kws['group'].timezone                       
        key = self.meetup_mapper.fro('time')
        kws[key] = fro_meetup_timestamp(kws[key],tzinfo)
        return kws
    
    def _post_object_to_meetup_data (self,obj,kws):
        kws['time'],_ = to_meetup_timestamp(kws['time'])
        kws['group_id'] = obj.group.id        
        raise NotImplementedError("not sure if what data meetup wants back to change this")
        return kws        

    def _post_object_creation_or_update (self,obj,md):
        venue = Venue.objects.from_meetup_data(md['venue'],sync=True)
        obj.venue.add(venue)
        return obj
     
    def past(self):
        return Event.objects.filter(status='past')

    def upcoming(self):
        return Event.objects.filter(status='upcoming')

    def pending(self):
        return Event.objects.filter(status='pending')
      
class Event(models.Model): 
    """ Meetup Event Model """
    STATUS_CHOICES = STATUS_CHOICES
    VISIBILITY_CHOICES = VISIBILITY_CHOICES
    objects = EventManager()
    
    # Meetup.com fields
    event_url = models.URLField(max_length=255, blank=True)        
    name = models.CharField(max_length=255, blank=True)    
    status = models.CharField(max_length=16, choices=[(s,s) for s in STATUS_CHOICES])
    visibility = models.CharField(max_length=16, choices=[(s,s) for s in VISIBILITY_CHOICES])
    description = models.TextField(blank=True)
    
    headcount = models.IntegerField(default=0,blank=True)
    yes_rsvp_count = models.IntegerField(default=0,blank=True)
    waitlist_count = models.IntegerField(default=0,blank=True)
    maybe_rsvp_count = models.IntegerField(default=0,blank=True)    
    
    event_timestamp = models.DateTimeField()    
    # created_timestamp
    
    venue = models.ManyToManyField(Venue)
    group = models.ForeignKey(Group)

    def __unicode__ (self):
        return self.name    
    
    def short_description(self, length=64):        
        desc = self.description[:length]
        if len(desc) == length:
            desc = desc[:-3] + "..."
        return desc    

    def set_view_tz (self,tz):
        if isinstance(tz,basestring):
            self._view_tz = pytz.timezone(tz)
        else:
            self._view_tz = tz 
    
    def get_view_tz (self):
        return getattr(self,'_view_tz',None)
      
    def _event_timestamp_in_view_tz (self,tz=None):
        dt = self.event_timestamp
        view_tz = self.get_view_tz()
        if tz is not None:  
            dt = dt.astimezone(tz)
        elif view_tz is not None:
            dt = dt.astimezone(view_tz)
        return dt        
    
    def view_when (self,tz=None):
        yr = self.view_year(tz)
        mo = self.view_month(tz)
        day = self.view_day(tz)
        t = self.view_time_of_day(tz)        
        when = mo
        if str(datetime.datetime.today().year) != yr:
            when += " "+yr
        when += " {} at {}".format(day,t)
        return when    
    
    def view_month (self,tz=None):
        """ Event timestamp month string """
        mo = {1:"January",
                  2:"Febuary",
                  3:"March",
                  4:"April",
                  5:"May",
                  6:"June",
                  7:"July",
                  8:"August",
                  9:"September",
                  10:"October",
                  11:"November",
                  12:"December"}                                            
        dt = self._event_timestamp_in_view_tz(tz)
        return mo[dt.month]
        
    def view_day (self,tz=None):
        """ Event timestamp day of month """ 
        return str(self._event_timestamp_in_view_tz(tz).day)

    def view_year (self,tz=None):
        """ Event timestamp day of month """ 
        return str(self._event_timestamp_in_view_tz(tz).year)

    def view_weekday (self,tz=None):
        """ Event timestamp day of week string """    
        wk = {0:"Monday",
                  1:"Tuesday",
                  2:"Wednesday",
                  3:"Thursday",
                  4:"Friday",
                  5:"Saturday",
                  6:"Sunday"}
        dt = self._event_timestamp_in_view_tz(tz)                 
        return wk[dt.weekday()]

    def view_time_of_day (self,hour24=False,tz=None):
        dt = self._event_timestamp_in_view_tz(tz)
        h = dt.hour
        m = dt.minute        
        when = "{}:{:02}"
        if not hour24:
            if h < 12:                
                when += " AM"
            else:
                h -= 12
                when += " PM"        
        return when.format(h,m)
        
    
    


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
PURPOSE: For testing meetup models
AUTHOR: dylangregersen
DATE: Mon Sep 15 00:52:58 2014
"""
# ########################################################################### #

# import modules 

from __future__ import print_function, division, unicode_literals
import os 
import unittest
from django.conf import settings
from meetup.api import MeetupClient

MEETUP_KEY =  settings.MEETUP_KEY

# ########################################################################### #


# TODO: implement tests

def test_basic_invoke ():
    """ make sure the invoke returns correct objects """
    pass
        
pass 
# ########################################################################### #
if __name__ == "__main__":
    pass
    



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
PURPOSE: methods for syncing meetup.models with api.meetup.com data 
AUTHOR: dylangregersen
DATE: Mon Sep 15 00:12:21 2014
"""
# ########################################################################### #

# import modules 

from __future__ import print_function, division, unicode_literals
from django.conf import settings
from meetup.api import MeetupClient
from meetup.models import Venue, Group, Event

MEETUP_KEY =  settings.MEETUP_KEY

# ########################################################################### #

def sync_group_events (group_id,client=None):
    """ Use meetup group id to sync all events to this data base """
    if client is None:
        client = MeetupClient(MEETUP_KEY)
    # ======================= get the group
    params = {}
    params['group_id'] = group_id
    results = client.invoke("/2/groups",params=params)['results']
    if not len(results):
        raise ValueError("No meetup group_id {}".format(params['group_id']))
    
    # ======================= sync events for the group    
    for group_data in results:
        group = Group.objects.from_meetup_data(group_data)
        print(" -- for group {} --".format(group.name))
        params['status'] = 'upcoming,past,cancelled'
        events = client.invoke("/2/events",params)['results']
        for event_data in events:                    
            event = Event.objects.from_meetup_data(event_data)
            print("   -- sync event {} --".format(event.name))                




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
PURPOSE: Utilities for converting to and from Meetup data 
AUTHOR: dylangregersen
DATE: Mon Sep 15 09:18:17 2014
"""
# ########################################################################### #

# import modules 

from __future__ import print_function, division, unicode_literals
import os 
import time
import datetime
import pytz

# ########################################################################### #

def fro_meetup_geo (geo):
    """ From Meetup Data to geo location"""
    if geo is None or not geo:
        return 
    return float(geo)
    
def to_meetup_geo (geo):
    """ from geo location to Meetup data geo """
    return geo
    
def fro_meetup_timestamp (t,tzinfo=""):
    """ Take time stamp from Meetup, convert to datetime 
    assumes utc if not tzinfo is given

    t : integer
        time in milliseconds
        
    """
    # convert to structured time
    struct_time = time.gmtime(int(t) / 1000.0)    
    # get datetime object
    keys = ['year','month','day','hour','minute','second']
    kws = {k:struct_time[i] for i,k in enumerate(keys)}
    kws['tzinfo'] = pytz.utc
    dt = datetime.datetime(**kws)
    # apply timezone info if needed
    if len(tzinfo):
        return dt.astimezone(pytz.timezone(tzinfo))
    else:
        return dt

def to_meetup_timestamp (ts):
    tzinfo = str(ts.tzinfo)
    t = time.mktime(d.timetuple())
    return t,tzinfo




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
PURPOSE: For testing meetup models
AUTHOR: dylangregersen
DATE: Mon Sep 15 00:52:58 2014
"""
# ########################################################################### #

# import modules 

from __future__ import print_function, division, unicode_literals
import os 
from meetup.models import Event,Group,Venue,Member
import unittest

# ########################################################################### #


# TODO: implement tests

def test_Event_meetup_data_to_kws ():
    # one for Event,Group,Venue,Member
    pass
    
def test_Event_object_to_meetup_data ():
    # one for Event,Group,Venue,Member
    pass
        
pass 
# ########################################################################### #
if __name__ == "__main__":
    pass
    



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
PURPOSE: Tools to access the Meetup.com API
AUTHOR: dylangregersen
DATE: Mon Sep 15 00:12:21 2014
"""
# ########################################################################### #

# import modules 

from __future__ import print_function, division, unicode_literals
import os
from urllib import urlencode
from urllib2 import urlopen
try:
    import json
except ImportError:
    import simplejson as json

# ########################################################################### #

class MeetupClient(object):
    """ MeetupClient """
    def __init__(self, api_key):
        """ Find your api_key from https://secure.meetup.com/meetup_api/key/"""
        self.api_key = api_key
    
    def invoke (self, meetup_method, params=None, method='GET'):
        """ For invoking a request
        
        Parameters
        meetup_method : string
            see http://www.meetup.com/meetup_api/docs/
        params : dict or None
            parameters passed to the request                    
        method : string  
            'GET' or 'POST'
        
        Returns
        response : dict
        
        """
        # get the parameters 
        params = params.copy() if params is not None else {}
        params['key'] = self.api_key
        params['page'] = 1000
                
        # the specific meetup method
        # see http://www.meetup.com/meetup_api/docs/
        if meetup_method.startswith("/"):
            meetup_method = meetup_method[1:]        
        url =  os.path.join("http://api.meetup.com",meetup_method)
        
        # get response
        if method == 'GET':
            return self._get(url, params)
        elif method == 'POST':
            return self._post(url, params)
    
    def _get(self, url, params):
        url = "{}?{}".format(url, urlencode(params))
        content = urlopen(url).read()
        content = unicode(content, 'utf-8', 'ignore')
        return json.loads(content)
        
    def _post(self, url, params):
        content = urlopen(url, urlencode(params)).read()
        content = unicode(content, 'utf-8', 'ignore')
        return json.load(content)
        
    def get_events (self,id,id_type="group",**params):
        params = params.copy()
        params.setdefault('status',"upcoming,past,pending")
        params['{}_id'.format(id_type)] = id
        return self.invoke("2/events",params)
        
    def update_event(self, event_id, **params):
        return self.invoke('event/{}'.format(event_id), params, method='POST')




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
PURPOSE: Install django-meetup 
AUTHOR: dylangregersen
DATE: Tue Sep 16 09:01:37 2014
"""
import os
import meetup
try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages

install_requires = open("requirements.txt").read().split("\n")
readme = open('README.rst').read()+"\nLicense\n-------\n"+open("LICENSE").read()

setup(
    name="django-meetup",
    version=meetup.__version__,
    packages=find_packages(),  
    include_package_data=True,  
    url="https://github.com/astrodsg/django-meetup",
    description="General purpose Django Meetup database to sync with Meetup.com.",
    long_description=readme,
    license="3-clause BSD style license",
    author="Dylan Gregersen",
    author_email = "gregersen.dylan@gmail.com",
    platforms=["any"],
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    install_requires=install_requires,    
)



from django.contrib import admin
from django.conf import settings
from meetup.models import Venue,Event,Group,Member

# class AccountAdmin(admin.ModelAdmin):
#     list_display = ('key','description','container_id','sync')
#     prepopulated_fields = {'slug': ('description',)}
# admin.site.register(Account, AccountAdmin)

class VenueAdmin (admin.ModelAdmin):
    list_display = ('id','name','address_1','city','state')

class GroupAdmin(admin.ModelAdmin):
    list_display = ('id','name','link','n_members')

class EventAdmin(admin.ModelAdmin):
    list_display = ('id','view_when','name','status','group','short_description')
    list_filter = ('status','group')

if getattr(settings,'MEETUP_ALLOW_ADMIN',False):
    admin.site.register(Venue, VenueAdmin)
    admin.site.register(Group, GroupAdmin)
    admin.site.register(Event, EventAdmin)        



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
PURPOSE: Sync Meetup group events to local database
AUTHOR: dylangregersen
DATE: Mon Sep 15 00:54:16 2014
"""
# ########################################################################### #

# import modules 

from __future__ import print_function, division, unicode_literals
import os 
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from meetup.sync import sync_group_events
from meetup.api import MeetupClient

MEETUP_KEY =  settings.MEETUP_KEY

# ########################################################################### #

class Command(BaseCommand):
    help = 'Sync Meetup group events to local database'

    def add_arguments(self, parser):
        parser.add_argument('group_id', nargs='+', type=int,help="group id, default is settings.MEETUP_GROUP_ID")
        parser.add_argument('--api_key',type=str,help="Key used for querying Meetup")        
                    
    def handle(self, *args, **options):
        client = MeetupClient(options.get('api_key',MEETUP_KEY))
        # ======================= get the group
        if len(args) == 1:
            group_id = int(args[0])
        else:        
            group_id = settings.MEETUP_GROUP_ID
        # ======================= sync events for the group            
        sync_group_events(group_id,client)
        


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
PURPOSE: Methods and functions to assist-viewing/view content
AUTHOR: dylangregersen
DATE: Tue Sep 16 08:40:49 2014
"""
# ########################################################################### #

from __future__ import print_function, division, unicode_literals
from django.conf import settings
from django.shortcuts import render
from meetup.models import Venue,Group,Event
from django.template import RequestContext

MEETUP_GROUP_ID = getattr(settings,"MEETUP_GROUP_ID",None)

# ########################################################################### #

def next_group_event (group=MEETUP_GROUP_ID,**filter):
    """ Recover the next upcoming event for a group
    
    default group is from ``django.conf.settings.MEETUP_GROUP_ID``. If None then
    you will have to give the group explicitly when calling this function
    
    Parameters
    ----------
    group : Group.object or Group.object.pk
    filter : dict
        Refine the Event.objects.filter(**filter) call
    
    Returns
    -------
    next : single Event.object or None
    
    """
    filter['status'] = 'upcoming'
    filter['group'] = Group.objects.get(pk=group)        
    events = Event.objects.filter(**filter).order_by("-event_timestamp")
    n = len(events)
    if n > 0:
        return events[n-1]

def view_next_event (template):
    def render_view (request):
        context = RequestContext(request)    
        context_dict = dict(next_group_event=next_group_event())
        return render_to_response(template,context_dict,context)

# NOTES:
# 
# * Filter events : the ``status`` field uses useful "upcoming","past","pending" keywords
# which you can filter by using Event.objects.filter(status="upcoming")
#

# ########################################################################### #
